/*! @file Hardware_Init.c
 * @brief This file implement the initialization of the hardware.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#include <stdint.h>
#include "string.h"
#include "stdio.h"
#include <zephyr/kernel.h>

#include "HAL_GPIO.h"
#include "HAL_SPI.h"
#include "Hardware_Init.h"

void pin_isr(	void )
{
     printk("Interrupt Detected \n");
}

//! Array of pin descriptors
static tGPIOPinDescriptor gpioDescriptorArray[HARDWARE_INIT_MAX_NUMBER_GPIO_PINS] =
{
  // 1. LED Blue
	{
	.pinIdentifier = HARDWARE_INIT_LED_BLUE_PIN_NUMBER,
	.portConfig = HARDWARE_INIT_LED_BLUE_PORT_NUMBER,
	.pinInputOutputConfig = HAL_GPIO_MODE_OUTPUT_PP,
	.pullConfig = HAL_GPIO_NOPULL,
	.callbackFunction = NULL,
  .defaultValue = HAL_GPIO_VALUE_LOW
	},
  // 2. LED Green
	{
	.pinIdentifier = HARDWARE_INIT_LED_GREEN_PIN_NUMBER,
	.portConfig = HARDWARE_INIT_LED_GREEN_PORT_NUMBER,
	.pinInputOutputConfig = HAL_GPIO_MODE_OUTPUT_PP,
	.pullConfig = HAL_GPIO_NOPULL,
	.callbackFunction = NULL,
  .defaultValue = HAL_GPIO_VALUE_LOW
	},
  // 3. LED Red
	{
	.pinIdentifier = HARDWARE_INIT_LED_RED_PIN_NUMBER,
	.portConfig = HARDWARE_INIT_LED_RED_PORT_NUMBER,
	.pinInputOutputConfig = HAL_GPIO_MODE_OUTPUT_PP,
	.pullConfig = HAL_GPIO_NOPULL,
	.callbackFunction = NULL,
  .defaultValue = HAL_GPIO_VALUE_LOW
	}
};

//! SPI pin descriptor
static tSPIDescriptor spiDescriptor = 
{
    .misoPinIdentifier = HARDWARE_INIT_SPI_MISO_PIN_NUMBER,
    .misoPortConfig = HARDWARE_INIT_SPI_MISO_PORT_NUMBER,
    .mosiPinIdentifier = HARDWARE_INIT_SPI_MOSI_PIN_NUMBER,
    .mosiPortConfig = HARDWARE_INIT_SPI_MOSI_PORT_NUMBER,
    .sclkPinIdentifier = HARDWARE_INIT_SPI_SCLK_PIN_NUMBER,
    .sclkPortConfig = HARDWARE_INIT_SPI_SCLK_PORT_NUMBER,
    .ccsPinIdentifier = HARDWARE_INIT_SPI_CS_PIN_NUMBER, 
    .ccsPortConfig = HARDWARE_INIT_SPI_CS_PORT_NUMBER, 
    .spiFrequency = HAL_SPI_FREQUENCY_1M,
    .spiClockPolarity = HAL_SPI_CPOL_LOW, 
    .spiClockPhase = HAL_SPI_CPHA_LOTOHI,
};

/*! This function initialize all the HAL configurations
 *
 */
void Hardware_Init (void)
{

  ABTC_HAL_GPIO_Config(                                                   \
                  &gpioDescriptorArray, 								                    \
                  HARDWARE_INIT_MAX_NUMBER_GPIO_PINS);

  ABTC_HAL_SPI_Init( &spiDescriptor );

}

/*! This function gives the required handlerType
 *  @param[in] ppDescriptor Pointer to the address in witch the descriptor 	\
 *  will be copied
 *  @param[in] descriptorType Descriptor type required according to the 	\
 *  enum eDescriptorType
 */
void Hardware_Init_Get_Descriptor( void **ppDescriptor, uint8_t descriptorType )
{
	if(descriptorType >= MAX_NUMBER_OF_DESCRIPTOR_TYPES)
  {
		return;
	}
	switch( descriptorType )
  {
		case HARDWARE_INIT_GPIO_DESCRIPTOR:
			*ppDescriptor = &gpioDescriptorArray;
			break;
    case HARDWARE_INIT_SPI_DESCRIPTOR:
      *ppDescriptor = &spiDescriptor;
      break;
		default:
			*ppDescriptor = NULL;
			return;
	}
}

