/*! @file Hardware_Init.h
 * @brief This file implement the initialization of the hardware.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#ifndef __HARDWARE_INIT_
#define __HARDWARE_INIT_

#include "stdint.h"

/* GPIO definition */
//! Port number of LED Blue
#define HARDWARE_INIT_LED_BLUE_PORT_NUMBER      0
//! Pin number of LED Blue
#define HARDWARE_INIT_LED_BLUE_PIN_NUMBER       9
//! Port number of LED Green
#define HARDWARE_INIT_LED_GREEN_PORT_NUMBER     0
//! Pin number of LED Green
#define HARDWARE_INIT_LED_GREEN_PIN_NUMBER      8
//! Port number of LED Red
#define HARDWARE_INIT_LED_RED_PORT_NUMBER       1
//! Pin number of LED Red
#define HARDWARE_INIT_LED_RED_PIN_NUMBER        3

/* SPI definition */
//! SPI MOSI PORT
#define HARDWARE_INIT_SPI_MOSI_PORT_NUMBER      1
//! SPI MOSI pin
#define HARDWARE_INIT_SPI_MOSI_PIN_NUMBER       5
//! SPI MISO PORT
#define HARDWARE_INIT_SPI_MISO_PORT_NUMBER      1
//! SPI MISO pin
#define HARDWARE_INIT_SPI_MISO_PIN_NUMBER       8
//! SPI SCLK PORT
#define HARDWARE_INIT_SPI_SCLK_PORT_NUMBER      1
//! SPI SCLK pin
#define HARDWARE_INIT_SPI_SCLK_PIN_NUMBER       6
//! SPI CS PORT
#define HARDWARE_INIT_SPI_CS_PORT_NUMBER        1
//! SPI CS pin
#define HARDWARE_INIT_SPI_CS_PIN_NUMBER         9

/*! @enum eGpioPins
 *  @brief Enumerator with the GPIO pins
 *  @typedef tGpioPins
 *  @brief Data type related to the enum eGpioPins
 */
typedef enum eGpioPins
{
    //! LED BLUE pin
    HARDWARE_INIT_GPIO_LED_BLUE,
    //! LED GREEN pin
    HARDWARE_INIT_GPIO_LED_GREEN,
    //! LED RED pin
    HARDWARE_INIT_GPIO_LED_RED,
    //! Max number of pins configured
    HARDWARE_INIT_MAX_NUMBER_GPIO_PINS
}tGpioPins;





/*!@enum eDescriptorType
 * @brief Enumerator with the hardware descriptor types
 * @typedef tDescriptorType
 * @tDescriptorType Data type associated with the enumeration eDescriptprType
 */
typedef enum eDescriptorType
{
	//! GPIO Handler
	HARDWARE_INIT_GPIO_DESCRIPTOR,
    //! SPI Descriptor 
    HARDWARE_INIT_SPI_DESCRIPTOR, 
	//! Max number oh handler types
	MAX_NUMBER_OF_DESCRIPTOR_TYPES,
}tDescriptorType;

/*! This function initialize all the HAL configurations
 *
 */
void Hardware_Init (void);

/*! This function gives the required handlerType
 *  @param[in] ppDescriptor Pointer to the address in witch the descriptor 	\
 *  will be copied
 *  @param[in] descriptorType Descriptor type required according to the 	\
 *  enum eDescriptorType
 */
void Hardware_Init_Get_Descriptor( void **ppDescriptor, uint8_t descriptorType);

#endif /* __HARDWARE_INIT_ */
