/*! @file BLE.c
 * @brief This file implement the BLE middleware
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████

*/

#include "Hardware_Init.h"
#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>
#include <zephyr/settings/settings.h>
#include "LEDs.h"
#include "BLE.h"

//! Name of BLE device. Defined in prj.conf
#define DEVICE_NAME             CONFIG_BT_DEVICE_NAME

//! Lenght of the BLE device's name
#define DEVICE_NAME_LEN         (sizeof(DEVICE_NAME) - 1)

//! Advertisement data structure
static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, DEVICE_NAME, DEVICE_NAME_LEN),
};
//! Device service
static const struct bt_data sd[] = {
	BT_DATA_BYTES(BT_DATA_UUID128_ALL, BLE_WALLY_CUSTOM_SERVICE ),
};
//! Indicates if the notifications on TX characteristics are enabled
static bool txNotificationsEnabled = false;

/*! This callback will be called when a new connection takes place
 * @param[in] pConnection Connection handler
 * @param[in] err Indicates if there was any error in the connection
 */
static void connected( struct bt_conn *pConnection, uint8_t err )
{
	if( err ){
		/* Connection failed ! */
		return;
	}

	/* Connection successful. Do here whatever needed when a new 
   succesful connection
  */

}

/*! This callback will be called when there is a disconnection event
 * @param[in] pConnection Connection handler
 * @param[in] reason Disconnection reason
 */
static void disconnected(struct bt_conn *pConnection, uint8_t reason)
{
	/* Device disconnected! Should we do something? */

}

BT_CONN_CB_DEFINE(conn_callbacks) = {
	.connected        = connected,
	.disconnected     = disconnected
};

/*! This callback is called each time the suscription to the TX notification
 * characteristic changes
 * @param[in] attr Pointer to the GATT attributes
 * @param[in] value GATT value
 */
static void BLE_Notify_TX_Changed(                                          \
                                const struct bt_gatt_attr *attr,            \
                                uint16_t value )
{
	txNotificationsEnabled = (value == BT_GATT_CCC_NOTIFY);
}

/*! Really, you don't need this, anyway just to show you how to do a callback
 * intended to be called anytime a characteristic is read
 * @param[in] pConnection Connection Handler
 * @param[in] pAttributes Handler of GATT attributes
 * @param[in] pData Pointer to the data buffer
 * @param[in] nData Number of bytes in the data pointed by pData
 * @param[in] offset Data offset
 * @return read value
 */
static ssize_t BLE_Wally_TX_Read(                                           \
                          struct bt_conn *pConnection,                      \
			                    const struct bt_gatt_attr *pAttributes,           \
			                    void *pData,                                      \
			                    uint16_t nData,                                   \
			                    uint16_t offset )
{
  /*
  The variable that we called here foo must be static, so it is better to be
  out of this function, this is provided only as an example.
  */
  /*
  static uint32_t foo;
	return bt_gatt_attr_read(                                               \
                          pConnection,                                      \
                          pAttributes,                                      \
                          pData,                                            \
                          nData,                                            \
                          offset,                                           \
                          &foo,                                             \
                          sizeof(uint32_t) );
  */
  return 0;
}

/*! Callback called each time a data is written on the RX BLE characteristic
 * @param[in] pConnection Connection Handler
 * @param[in] pAttributes Handler of GATT attributes
 * @param[in] pData Pointer to the data buffer
 * @param[in] nData Number of bytes in the data pointed by pData
 * @param[in] offset Data offset
 * @return read value
 */
static ssize_t BLE_Wally_RX_Write(                                          \
                          struct bt_conn *pConnection,                      \
			                    const struct bt_gatt_attr *pAttributes,           \
			                    const void *pData,                                \
			                    uint16_t nData,                                   \
			                    uint16_t offset,                                  \
                          uint8_t flags )
{
  
  if( ( NULL == pData ) || ( NULL == pConnection ) || ( NULL == pAttributes ) ){
		return BT_GATT_ERR(BT_ATT_ERR_INVALID_HANDLE);
	}
	/* The data is received in the buffer pData and its legth is nData */
	
  // Do whatever is needed here. This example is just an echo
  BLE_Wally_TX( pData, nData );
  
  uint16_t i = 0;
  while( i++ < 2000 ){
    if( (i & 0xFF) < 7 ){
      LED_On( LEDS_RED );
    }
    else{
      LED_Off( LEDS_RED );
    }
    if( ( i & 0xFF ) < 0x57 ){
      LED_On( LEDS_GREEN );
    }
    else{
      LED_Off( LEDS_GREEN );
    }
    if( ( i & 0xFF ) < 0x5B ){
      LED_On( LEDS_BLUE );
    }
    else{
      LED_Off( LEDS_BLUE );
    }
    k_msleep(1);
  }

	return nData;
}

/* Walle BLE Service Declaration */
BT_GATT_SERVICE_DEFINE(WallyService,
BT_GATT_PRIMARY_SERVICE(BLE_WALLY_CUSTOM_SERVICE_UUID),
	BT_GATT_CHARACTERISTIC(BLE_WALLY_TX_UUID,
			      BT_GATT_CHRC_READ | BT_GATT_CHRC_NOTIFY,
			      BT_GATT_PERM_READ, BLE_Wally_TX_Read, NULL,
			      NULL),
    BT_GATT_CCC(BLE_Notify_TX_Changed,
		    BT_GATT_PERM_READ | BT_GATT_PERM_WRITE),
    BT_GATT_CHARACTERISTIC(BLE_WALLY_RX_UUID,
			       BT_GATT_CHRC_WRITE,
			       BT_GATT_PERM_WRITE, NULL, BLE_Wally_RX_Write,
			       NULL),
);

/*! This function initializes the BLE functionality and starts advertising
 * @return 0 When success. A negative value when error
 */
int BLE_Init( void )
{
  int err;

  err = bt_enable( NULL );
	if( err ) {
		// BT init failed. Reason: err
		return err;
	}

	if (IS_ENABLED(CONFIG_SETTINGS)) {
		settings_load();
	}

	err = bt_le_adv_start(                                                    \
                      BT_LE_ADV_CONN,                                       \
                      ad,                                                   \
                      ARRAY_SIZE(ad),                                       \
                      sd,                                                   \
                      ARRAY_SIZE(sd) );
	if( err ){
		// Advertising failed to start. Reason: err
		return err;
	}

  return err;
}

/*! This function is used for sending a notification on the Wally TX 
 * characteristic
 * @param[in] pData Pointer to the data where the data to be notified should be
 * @param[in] nData Number of data, in bytes, in the array pointed by pData
 * @return 1 when success, 0 when error
*/
uint8_t BLE_Wally_TX( const uint8_t *pData, uint8_t nData )
{
	if( ( NULL == pData ) || ( 0 == nData ) ){
    return 0;
  }
  if( false == txNotificationsEnabled ){
    // Notifications are not enabled. We can't notify
    return 0;
  }

	if( bt_gatt_notify(	NULL, &WallyService.attrs[ 2 ], pData, nData ) >= 0 ){
		return 1;
	}
	return 0;
}