/*! @file BLE.h
 * @brief This is the header file of the BLE
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████

*/

#ifndef __BLE_H_
#define __BLE_H___

#include <stdint.h>

/* BLE Custom service definitions */

//! Definition for encoding the BLE Wally Service
#define BLE_WALLY_CUSTOM_SERVICE                                            \
	BT_UUID_128_ENCODE(0x00005500, 0xCAFE, 0x0FEA, 0x2311, 0xCACADEBACA)
//! UUID of the BLE Wally service
#define BLE_WALLY_CUSTOM_SERVICE_UUID                                       \
    BT_UUID_DECLARE_128(BLE_WALLY_CUSTOM_SERVICE)


/* Device peripheral characteristics definitions */

//! Definition for encoding the TX characteristic
#define BLE_WALLY_TX_CHAR                                                   \
    BT_UUID_128_ENCODE(0x00002500, 0xCAFE, 0x0FEA, 0x2311, 0xCACADEBACA)
//! UUID for the TX characteristic
#define BLE_WALLY_TX_UUID             BT_UUID_DECLARE_128(BLE_WALLY_TX_CHAR)

//! Definition for encoding the RX characteristic
#define BLE_WALLY_RX_CHAR                                                   \
    BT_UUID_128_ENCODE(0x00002501, 0xCAFE, 0x0FEA, 0x2311, 0x25E15A69964)
//! UUID for the Accelerometer Y characteristic
#define BLE_WALLY_RX_UUID             BT_UUID_DECLARE_128(BLE_WALLY_RX_CHAR)

/*! This function initializes the BLE functionality and starts advertising
 * @return 0 When success. A negative value when error
 */
int BLE_Init( void );

/*! This function is used for sending a notification on the Wally TX 
 * characteristic
 * @param[in] pData Pointer to the data where the data to be notified should be
 * @param[in] nData Number of data, in bytes, in the array pointed by pData
 * @return 1 when success, 0 when error
*/
uint8_t BLE_Wally_TX( const uint8_t *pData, uint8_t nData );

#endif /* __BLE_H_ */

