/*! @file GPIO.c
 * @brief This file implement the middleware related to the GPIO 			\
 *  and the definition of the interruption function of GPIO pins.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#include <stdint.h>
#include "Hardware_Init.h"
#include "GPIO.h"

//! GPIO pins descriptor
static tGPIOPinDescriptor *pGpioDescriptor;

/*! This function get the GPIO Handler
 */
void GPIO_Init( void )
{
	Hardware_Init_Get_Descriptor((void **)&pGpioDescriptor,				            \
			                  HARDWARE_INIT_GPIO_DESCRIPTOR);

}

/*! This function sets a digital output to 0 or 1
 *  @param[in] gpioPin 	Pin to set the digital output
 *  @param[in] value	Output value to be set
 */
void GPIO_Set_Output( 	uint8_t gpioPin,									                  \
						uint8_t value)
{
	ABTC_HAL_GPIO_Set_Output( &pGpioDescriptor[ gpioPin ], value );
}

/*! This function gets a digital input
 * @param[in] gpioPin 	Pin to read the input
 * @return Read value
 */
uint8_t GPIO_Read_Input( uint8_t gpioPin )
{
	uint8_t readValue = 0;

	readValue = ABTC_HAL_GPIO_Read_Input((char *)pGpioDescriptor + 			\
			gpioPin*sizeof(tGPIOPinDescriptor));

	return readValue;
}

/*! This function implements the register of the gpio interrupt function
 *  @brief pInterruptFunction Interrupt function for the gpio pin
 */
void GPIO_Set_Interrupt_Function( 	uint8_t gpioPin,						\
									void (* pInterruptFunction)(void))
{
	ABTC_HAL_GPIO_Set_Interrupt((char *)pGpioDescriptor + 					\
			gpioPin*sizeof(tGPIOPinDescriptor), 							\
			pInterruptFunction);
}
