/*! @file GPIO.h
 * @brief This file implement the middleware related to the GPIO 			\
 *  and the definition of the interruption function of GPIO pins.
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#ifndef _GPIO_H_
#define _GPIO_H_

#include <stdint.h>
#include "Hardware_Init.h"
#include "HAL_GPIO.h"

//! Wrapper values from GPIO_VALUE to HAL_GPIO_VALUE
#define GPIO_VALUE_HIGH       HAL_GPIO_VALUE_HIGH
//! Wrapper values from GPIO_VALUE to HAL_GPIO_VALUE
#define GPIO_VALUE_LOW       HAL_GPIO_VALUE_LOW

/*! @enum eStatePin
 *  @brief Enumeration with the different state of the pins
 *  @typedef tStatePin
 *  @brief Data type related to the enum eStatePin
 */
typedef enum eStatePin{
	//! State Low
	GPIO_PIN_LOW,
	//! State High
	GPIO_PIN_HIGH
}tStatePin;

/*! This function get the GPIO Handler
 */
void GPIO_Init( void );

/*! This function sets a digital output to 0 or 1
 *  @param[in] gpioPin 	Pin to set the digital output
 *  @param[in] value	Output value to be set
 */
void GPIO_Set_Output( 	uint8_t gpioPin,									\
						uint8_t value);

/*! This function gets a digital input
 * @param[in] gpioPin 	Pin to read the input
 * @return Read value
 */
uint8_t GPIO_Read_Input( uint8_t gpioPin );

/*! This function implements the register of the gpio interrupt function
 *  @brief pInterruptFunction Interrupt function for the gpio pin
 */
void GPIO_Set_Interrupt_Function( 	uint8_t gpioPin,						\
									void (* pInterruptFunction)(void));

#endif /* _GPIO_ */