/*! @file LEDs.c
 * @brief This file implements the functions for dealing with the
 * board LEDs
 *
 * @author A BLUE THING IN THE CLOUD S.L.U.
 *    ===  When the technology becomes art ===
 *
 * http://abluethinginthecloud.com
 * j.longares@abluethinginthecloud
 *
 * (c) A BLUE THING IN THE CLOUD S.L.U.
 *
 *
 *

        ██████████████    ██    ██    ██  ██████    ██████████████
        ██          ██      ████████████████  ██    ██          ██
        ██  ██████  ██  ██████  ██    ██        ██  ██  ██████  ██
        ██  ██████  ██    ██████    ██      ██      ██  ██████  ██
        ██  ██████  ██      ██      ████  ██████    ██  ██████  ██
        ██          ██    ██      ██████    ████    ██          ██
        ██████████████  ██  ██  ██  ██  ██  ██  ██  ██████████████
                        ██████  ████  ██████  ████
        ██████  ██████████  ████    ████████      ████      ██
        ██  ████  ██    ██  ████        ████    ████████  ██    ██
            ██  ██  ████  ██      ██      ██      ██  ████  ██████
        ████  ████    ██      ██          ████  ██  ██        ██
            ██████████          ██      ██    ██  ████    ██  ████
          ██  ████    ██      ██████    ██  ██████████    ██    ██
        ██  ████  ████████████████  ██    ██        ████████  ████
                ████        ██  ██████  ██████████      ████  ██
        ██████  ████████████████    ████  ██    ██████    ██  ████
            ████████  ██████  ██    ██████      ██        ████  ██
        ██    ██  ████████  ██    ██        ██    ██          ████
          ████  ████          ██      ████████████  ██  ████  ██
        ██  ██████  ████  ██    ██      ████    ██████████
                        ██    ██████    ██      ██      ██  ██████
        ██████████████  ██  ██████  ██  ████  ████  ██  ████  ████
        ██          ██  ██      ████████  ██    ██      ████  ████
        ██  ██████  ██  ████  ██    ██████      ██████████    ████
        ██  ██████  ██    ██████    ██  ██  ████      ████  ██████
        ██  ██████  ██  ████      ██    ████  ██        ████    ██
        ██          ██  ██    ██      ██████████████  ██      ██
        ██████████████  ██████  ██        ██  ████    ██████  ████



*/

#include "Hardware_Init.h"
#include "GPIO.h"
#include "LEDs.h"

/*! Function for switching a LED on
* @param[in] LED led to be switched on
*/
void LED_On( tLED LED ){
    switch( LED ){
        case LEDS_BLUE:
            GPIO_Set_Output(HARDWARE_INIT_GPIO_LED_BLUE, GPIO_VALUE_HIGH);
        break;
        case LEDS_GREEN:
            GPIO_Set_Output(HARDWARE_INIT_GPIO_LED_GREEN, GPIO_VALUE_HIGH);
        break;
        case LEDS_RED:
            GPIO_Set_Output(HARDWARE_INIT_GPIO_LED_RED, GPIO_VALUE_HIGH);
        break;
        default:
            return;
    }
}

/*! Function for switching a LED off
* @param[in] LED led to be switched off
*/
void LED_Off( tLED LED ){
    switch( LED ){
        case LEDS_BLUE:
            GPIO_Set_Output(HARDWARE_INIT_GPIO_LED_BLUE, GPIO_VALUE_LOW);
        break;
        case LEDS_GREEN:
            GPIO_Set_Output(HARDWARE_INIT_GPIO_LED_GREEN, GPIO_VALUE_LOW);
        break;
        case LEDS_RED:
            GPIO_Set_Output(HARDWARE_INIT_GPIO_LED_RED, GPIO_VALUE_LOW);
        break;
        default:
            return;
    }
}

/*! Function used for toggling a LED
 * @param[in] LED LED to be toggled
 */
void LED_Toggle( tLED LED )
{
    uint8_t gpioPin;

    switch( LED ){
        case LEDS_BLUE:
            gpioPin = HARDWARE_INIT_GPIO_LED_BLUE;
        break;
        case LEDS_GREEN:
            gpioPin = HARDWARE_INIT_GPIO_LED_GREEN;
        break;
        case LEDS_RED:
            gpioPin = HARDWARE_INIT_GPIO_LED_RED;
        break;
        default:
            return;
    }
    if( GPIO_Read_Input( gpioPin ) == GPIO_VALUE_LOW ){
        GPIO_Set_Output( gpioPin, GPIO_VALUE_HIGH );
    }
    else{
        GPIO_Set_Output( gpioPin, GPIO_VALUE_LOW );
    }
}